﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;
public enum GameState { Playing, Paused, Lost, Won };
public class GameController : MonoBehaviour
{

    /// <summary>
    /// Important game object needed for game logic
    /// </summary>
    public GameController gameController;
    public GameObject player;
    public PlayerController playerScript;
    public GameObject boss;
    public float fixedEnemyDamage;
    public float backgroundSpeed;
    public bool paused;
    public ParticleSystem bossSpawnParticle;
    public GameObject bossUI;
    public GameState gameState;

    /// <summary>
    /// Score objects
    /// </summary>
    public int score;
    public int highScore;

    /// <summary>
    /// Game timers
    /// </summary>
    public float elapsedTime;
    public float time;

    /// <summary>
    /// Volume values
    /// </summary>
    public float musicVolume;
    public float soundEffectsVolume;

    /// <summary>
    /// Main HUD    Score,Timer,Player HealthBar
    /// </summary>
    public GameObject healthBar;
    public Text timerText;
    public Text scoreText;

    /// <summary>
    /// Different menus and buttons
    /// </summary>
    public Transform pauseMenu;
    public Transform loseMenu;
    public Transform winMenu;
    public Transform loseMenuButton;
    public Transform pauseMenuButton;
    public Transform winMenuButton;
    public Text winScoreText;
    public Text winHighScoreText;
    /// <summary>
    /// Level Music
    /// </summary>
    public AudioSource mainTheme;
    public AudioSource bossTheme;

    // Use this for initialization
    void Start()
    {
        //Get GameControllerScript
        InitGameController();
        //Get Player Object
        GameObject playerObject = GameObject.FindWithTag("Player");
        if (playerObject != null)
        {
            player = playerObject;
            playerScript = player.GetComponent<PlayerController>();
        }
        else
        {
            Debug.Log("Cannot find 'Player' object");
        }

        healthBar = GameObject.Find("HealthBar");

        ///GetHighscore from playerprefs
        if (PlayerPrefs.HasKey("HighScore"))
        {
            highScore = (int)PlayerPrefs.GetFloat("HighScore");
        }
        else
        {
            PlayerPrefs.SetFloat("HighScore", 0.0f);
            highScore = 0;
        }
        if (PlayerPrefs.HasKey("MusicVolume"))
        {
            AudioListener.volume = PlayerPrefs.GetFloat("MusicVolume");
        }
        else
        {
            AudioListener.volume = 0.8f;
        }
        mainTheme.Play();
        Play();

        
    }

    // Update is called once per frame
    void Update()
    {

        if (playerScript.healthPoints <= 0.0f && gameState == GameState.Playing)
            Lose();

        if (Input.GetButtonDown("Cancel") && gameState == GameState.Playing)
            Pause();

        else if (Input.GetButtonDown("Cancel") && gameState == GameState.Paused)
            Play();

        if (boss == null && gameState == GameState.Playing)
            Win();
    }
    private void FixedUpdate()
    {
        UpdateHud();

    }
    /// <summary>
    /// Find and initialize gameController script
    /// </summary>
    private void InitGameController()
    {
        GameObject gameControllerObject = GameObject.FindWithTag("GameController");
        if (gameControllerObject != null)
        {
            gameController = gameControllerObject.GetComponent<GameController>();
        }
        if (gameController == null)
        {
            Debug.Log("Cannot find 'GameController' script");
        }

    }
    /// <summary>
    /// Adss points to score
    /// </summary>
    /// <param name="points"></param>
    public void AddScore(int points)
    {
        score += points;

    }
    public int GetScore()
    {
        return score;
    }
    public GameObject GetPlayer()
    {
        return player;
    }
    public float GetElapsedTime()
    {
        return elapsedTime;
    }
    /// <summary>
    /// Update HUD elements such as Score,Time,Player HP
    /// </summary>
    public void UpdateHud()
    {
        //Update HealthBar
        healthBar.GetComponent<UnityEngine.UI.Scrollbar>().size = playerScript.healthPoints / playerScript.totalHealthPoints;
        //Update ScoreBar
        if (scoreText != null)
            scoreText.text = "Score: " + score.ToString();
        //Update TimeBar
        elapsedTime += Time.deltaTime;
        if (timerText != null)
            timerText.text = string.Format("{0:00} : {1:00} ", (int)elapsedTime / 60, (int)elapsedTime % 60);
    }
    /// <summary>
    /// Pause the game,activate pause menu
    /// </summary>
    public void Pause()
    {
        gameState = GameState.Paused;
        Time.timeScale = 0;
        pauseMenu.gameObject.SetActive(true);
        EventSystem.current.SetSelectedGameObject(pauseMenuButton.gameObject);
    }
    /// <summary>
    /// Play/resume game
    /// </summary>
    public void Play()
    {
        gameState = GameState.Playing;
        Time.timeScale = 1;
        pauseMenu.gameObject.SetActive(false);
        loseMenu.gameObject.SetActive(false);
        winMenu.gameObject.SetActive(false);
    }
    /// <summary>
    /// Lose game, activate lose menu
    /// </summary>
    public void Lose()
    {
        gameState = GameState.Lost;
        Time.timeScale = 0;
        loseMenu.gameObject.SetActive(true);
        EventSystem.current.SetSelectedGameObject(loseMenuButton.gameObject);
    }
    /// <summary>
    /// WIn game, activate win menu,and compute and save highscore
    /// </summary>
    public void Win()
    {
        gameState = GameState.Won;
        Time.timeScale = 0;
        winMenu.gameObject.SetActive(true);
        if (highScore < score)
        {
            highScore = score;
            PlayerPrefs.SetFloat("HighScore", highScore);
        }

        winScoreText.text = winScoreText.text + score.ToString();
        winHighScoreText.text = winHighScoreText.text + highScore.ToString();
        EventSystem.current.SetSelectedGameObject(winMenuButton.gameObject);
        PlayerPrefs.Save();
    }
    /// <summary>
    /// Activate boss spawn particles
    /// </summary>
    public void ActivateBoss()
    {
        Instantiate(bossSpawnParticle);
        /*if (!boss.activeSelf)
            Invoke("boss.SetActive(true)", 1.0f);*/
        //boss.SetActive(true);
    }
    /// <summary>
    /// Play main music theme while stopping boss theme
    /// </summary>
    public void PlayMainTheme()
    {
        bossTheme.Stop();
        mainTheme.Play();
    }
    /// <summary>
    /// Play boss music theme while stopping main theme
    /// </summary>
    public void PlayBossTheme()
    {
        mainTheme.Stop();
        bossTheme.Play();
    }
    /// <summary>
    /// Stops all music themes from playing
    /// </summary>
    public void StopMusic()
    {
        mainTheme.Stop();
        bossTheme.Stop();
    }
}